@@ -1,5 +1,5 @@
 /* This file is part of GNU Dico. 
-   Copyright (C) 1998, 1999, 2000, 2008, 2010 Sergey Poznyakoff
+   Copyright (C) 1998-2019 Sergey Poznyakoff
 
    GNU Dico is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -131,7 +131,7 @@ dict_auth(struct dict_connection *conn, dico_url_t url)
 }
 
 char *
-get_homedir()
+get_homedir(void)
 {
     char *homedir = getenv("HOME");
     if (!homedir) {
@@ -269,48 +269,126 @@ dict_transcript(struct dict_connection *conn, int state)
     }
 }
 
+static char const *
+urlstr(dico_url_t url)
+{
+    if (!url->string) {
+	if (!url->proto)
+	    xdico_assign_string(&url->proto, "dict");
+	if (!url->port)
+	    xdico_assign_string(&url->port, DICO_DICT_PORT_STR);
+
+	if (url->host) {
+	    asprintf(&url->string, "%s://%s:%s",
+		     url->proto,
+		     url->host,
+		     url->port);
+	} else {
+	    asprintf(&url->string, "%s:///%s", url->proto, url->path);
+	}
+    }   
+    
+    return url->string;
+}
+
 int
 dict_connect(struct dict_connection **pconn, dico_url_t url)
 {
-    struct sockaddr_in s;
-    int fd;
-    IPADDR ip;
+    int fd, rc, family;
+    struct addrinfo hints, *res, *rp;
     dico_stream_t str;
     struct dict_connection *conn;
+    char const *port = url->port ? url->port : DICO_DICT_PORT_STR;
     
-    XDICO_DEBUG_F2(1, _("Connecting to %s:%d\n"), url->host,
-		   url->port ? url->port : DICO_DICT_PORT);
-    fd = socket(PF_INET, SOCK_STREAM, 0);
-    if (fd == -1) {
-	dico_log(L_ERR, errno,
-		 _("cannot create dict socket"));
-	return 1;
+    XDICO_DEBUG_F1(1, _("Connecting to %s\n"), urlstr (url));
+
+    if (source_addr) {
+	memset(&hints, 0, sizeof(hints));
+	hints.ai_socktype = SOCK_STREAM;
+	rc = getaddrinfo(source_addr, NULL, &hints, &res);
+	if (rc) {
+	    dico_log(L_ERR, 0,
+		     _("bad source address: %s"), gai_strerror(rc));
+	    return 1;
+	}
+    
+	for (rp = res; rp; rp = rp->ai_next) {
+	    fd = socket(rp->ai_family, rp->ai_socktype, rp->ai_protocol);
+	    if (fd == -1)
+		continue;
+	    if (bind(fd, rp->ai_addr, rp->ai_addrlen) == 0)
+		break;
+	    close(fd);
+	}
+	
+	if (!rp) {
+	    dico_log(L_ERR, 0,
+		     _("can't bind to the source address"));
+	    return 1;
+	}
+    } else {
+	fd = -1;
     }
 
-    s.sin_family = AF_INET;
-    s.sin_addr.s_addr = htonl(source_addr);
-    s.sin_port = 0;
-    if (bind(fd, (struct sockaddr*) &s, sizeof(s)) < 0) {
-	dico_log(L_ERR, errno,
-		 _("cannot bind AUTH socket"));
+    memset(&hints, 0, sizeof(hints));
+    hints.ai_socktype = SOCK_STREAM;
+    if (url->host) {
+	rc = getaddrinfo(url->host, port, &hints, &res);
+	if (rc) {
+	    dico_log(L_ERR, 0,
+		     _("%s: can't get address: %s"),
+		     url->host, gai_strerror(rc));
+	    return -1;
+	}
+    } else {
+	struct sockaddr_un *s;
+
+	if (strlen(url->path) >= sizeof s->sun_path) {
+	    dico_log(L_ERR, 0, _("%s: UNIX socket name too long"), url->path);
+	    return -1;
+	}
+	
+	hints.ai_family = AF_UNIX;
+	hints.ai_addrlen = sizeof(struct sockaddr_un);
+	
+	s = xcalloc(1, hints.ai_addrlen);
+	s->sun_family = AF_UNIX;
+	strcpy(s->sun_path, url->path);
+
+	hints.ai_addr = (struct sockaddr *)s;
+
+	res = &hints;
     }
+    
+    for (rp = res; rp; rp = rp->ai_next) {
+	if (fd != -1 && family != rp->ai_family) {
+	    close(fd);
+	    fd = -1;
+	}
+	if (fd == -1) {
+	    family = rp->ai_family;
+	    fd = socket(family, SOCK_STREAM, 0);
+	    if (fd == -1) {
+		dico_log(L_ERR, errno,
+			 _("cannot create dict socket"));
+		continue;
+	    }
+	}
 
-    ip = get_ipaddr(url->host);
-    if (ip == 0) {
-	dico_log(L_ERR, 0, _("%s: Invalid IP or unknown host name"),
-		 url->host);
-	return 1;
+	if (connect(fd, rp->ai_addr, rp->ai_addrlen) != -1)
+	    break;
     }
-    s.sin_addr.s_addr = htonl(ip);
-    s.sin_port = htons(url->port ? url->port : DICO_DICT_PORT);
-    if (connect(fd, (struct sockaddr*) &s, sizeof(s)) == -1) {
-	dico_log(L_ERR, errno,
-		 _("cannot connect to DICT server %s:%d"),
-		 url->host, url->port ? url->port : DICO_DICT_PORT);
-	close(fd);
+    
+    if (!rp) {
+	dico_log(L_ERR, 0, _("%s: cannot connect"), urlstr(url));
 	return 1;
     }
 
+    if (res == &hints)
+	free(res->ai_addr);
+    else
+	freeaddrinfo(res);
+    
     if ((str = dico_fd_io_stream_create(fd, fd)) == NULL) {
 	dico_log(L_ERR, errno,
 		 _("cannot create dict stream: %s"),
@@ -511,25 +589,24 @@ _result_parse_def(struct dict_result *res)
     char *p;
     size_t i;
     struct define_result *def = xcalloc(res->count, sizeof(*def));
-    xdico_input_t input = xdico_tokenize_begin();
+    struct dico_tokbuf tb;
+
+    dico_tokenize_begin(&tb);
     
     res->set.def = def;
     p = res->base;
     for (i = 0; i < res->count; i++, def++) {
-	int argc;
-	char **argv;
-
-	/* FIXME: Provide a destructive version of xdico_tokenize_input */
-	xdico_tokenize_input(input, p, &argc, &argv);
-	def->word = xstrdup(argv[1]);
-	def->database = xstrdup(argv[2]);
-	def->descr = xstrdup(argv[3]);
+	/* FIXME: Provide a destructive version of xdico_tokenize_string? */
+	xdico_tokenize_string(&tb, p);
+	def->word = xstrdup(tb.tb_tokv[1]);
+	def->database = xstrdup(tb.tb_tokv[2]);
+	def->descr = xstrdup(tb.tb_tokv[3]);
 	p += strlen(p) + 1;
 	def->defn = p;
 	def->nlines = count_lines(p);
 	p += strlen(p) + 1;
     }
-    xdico_tokenize_end(&input);
+    dico_tokenize_end(&tb);
 }
 
 static void
