@@ -1,5 +1,5 @@
 /* This file is part of GNU Dico.
-   Copyright (C) 2012-2018 Sergey Poznyakoff
+   Copyright (C) 2012-2019 Sergey Poznyakoff
 
    GNU Dico is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -14,9 +14,7 @@
    You should have received a copy of the GNU General Public License
    along with GNU Dico.  If not, see <http://www.gnu.org/licenses/>. */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+#include <config.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <sys/types.h>
@@ -204,7 +202,7 @@ gcide_open_idx(struct gcide_db *db)
     
     idxname = dico_full_file_name(db->idx_dir, "GCIDE.IDX");
     if (!idxname) {
-	dico_log(L_ERR, errno, "gcide_open_idx");
+        DICO_LOG_MEMERR();
 	return 1;
     }
     
@@ -223,7 +221,7 @@ gcide_open_idx(struct gcide_db *db)
 }
 
 static dico_handle_t
-gcide_init_db(const char *dbname, int argc, char **argv, void *extra)
+gcide_init_db(const char *dbname, int argc, char **argv)
 {
     char *db_dir = NULL;
     char *idx_dir = NULL;
@@ -237,8 +235,10 @@ gcide_init_db(const char *dbname, int argc, char **argv, void *extra)
 	{ DICO_OPTSTR(idxdir), dico_opt_string, &idx_dir },
 	{ DICO_OPTSTR(index-program), dico_opt_string, &idxgcide },
 	{ DICO_OPTSTR(index-cache-size), dico_opt_long, &idx_cache_size },
-	{ DICO_OPTSTR(suppress-pr), dico_opt_bitmask, &flags, .v.value = GCIDE_NOPR },
-	{ DICO_OPTSTR(debug-lex), dico_opt_bitmask, &flags, .v.value = GCIDE_DBGLEX },
+	{ DICO_OPTSTR(suppress-pr), dico_opt_bitmask, &flags, 
+          .v.value = GCIDE_NOPR },
+	{ DICO_OPTSTR(debug-lex), dico_opt_bitmask, &flags, 
+          .v.value = GCIDE_DBGLEX },
 	{ NULL }
     };
     
@@ -252,7 +252,7 @@ gcide_init_db(const char *dbname, int argc, char **argv, void *extra)
     if (!idx_dir) {
 	idx_dir = strdup(db_dir);
 	if (!idx_dir) {
-	    dico_log(L_ERR, errno, "gcide_init_db");
+            DICO_LOG_ERRNO();
 	    free(db_dir);
 	    return NULL;
 	}
@@ -260,9 +260,9 @@ gcide_init_db(const char *dbname, int argc, char **argv, void *extra)
 
     db = calloc(1, sizeof(*db));
     if (!db) {
+        DICO_LOG_ERRNO();
 	free(db_dir);
 	free(idx_dir);
-	dico_log(L_ERR, ENOMEM, "gcide_init_db");
 	return NULL;
     }
     db->db_dir = db_dir;
@@ -431,7 +431,7 @@ find_matcher(const char *strat)
 }
 
 static int
-compare_ref(const void *a, void *b)
+compare_ref(const void *a, const void *b, void *closure)
 {
     struct gcide_ref const *aref = a;
     struct gcide_ref const *bref = b;
@@ -455,12 +455,11 @@ gcide_create_result_list(int unique)
     
     list = dico_list_create();
     if (!list) {
-	dico_log(L_ERR, ENOMEM, "gcide_create_result_list");
+        DICO_LOG_ERRNO();
 	return NULL;
     }
     if (unique) {
-	dico_list_set_comparator(list,
-				 (int (*)(const void *, void *)) compare_ref);
+	dico_list_set_comparator(list, compare_ref, NULL);
 	dico_list_set_flags(list, DICO_LIST_COMPARE_TAIL);
     }
     dico_list_set_free_item(list, free_ref, NULL);
@@ -472,14 +471,14 @@ gcide_result_list_append(dico_list_t list, struct gcide_ref *ref)
 {
     struct gcide_ref *copy = calloc(1,sizeof(*copy));
     if (!copy) {
-	dico_log(L_ERR, errno, "gcide_result_list_append");
+        DICO_LOG_ERRNO();       
 	return -1;
     }
     *copy = *ref;
     copy->ref_headword = strdup(ref->ref_headword);
     if (!copy->ref_headword ||
 	(dico_list_append(list, copy) && errno == ENOMEM)) {
-	dico_log(L_ERR, errno, "gcide_result_list_append");
+        DICO_LOG_ERRNO();
 	free(copy);
 	return -1;
     }
@@ -516,7 +515,7 @@ gcide_match_all(struct gcide_db *db, const dico_strategy_t strat,
 	return NULL;
     
     if (dico_key_init(&clos.key, strat, word)) {
-	dico_log(L_ERR, 0, _("gcide_match_all: key initialization failed"));
+	dico_log(L_ERR, 0, _("%s: key initialization failed"), __func__);
 	dico_list_destroy(&clos.list);
 	return NULL;
     }
@@ -533,7 +532,7 @@ gcide_match_all(struct gcide_db *db, const dico_strategy_t strat,
     
     res = calloc(1, sizeof(*res));
     if (!res) {
-	dico_log(L_ERR, errno, "gcide_match_all");
+        DICO_LOG_ERRNO();
 	dico_list_destroy(&clos.list);
     } else {
 	res->type = result_match;
@@ -559,7 +558,7 @@ gcide_match(dico_handle_t hp, const dico_strategy_t strat, const char *word)
     if (itr) {
 	res = calloc(1, sizeof(*res));
 	if (!res) {
-	    dico_log(L_ERR, errno, "gcide_match");
+            DICO_LOG_ERRNO();
 	    gcide_iterator_free(itr);
 	    return NULL;
 	}
@@ -593,7 +592,7 @@ gcide_define(dico_handle_t hp, const char *word)
     if (itr) {
 	res = calloc(1, sizeof(*res));
 	if (!res) {
-	    dico_log(L_ERR, errno, "gcide_define");
+            DICO_LOG_ERRNO();
 	    gcide_iterator_free(itr);
 	    return NULL;
 	}
@@ -749,7 +748,7 @@ output_def(dico_stream_t str, struct gcide_db *db, struct gcide_ref *ref)
 
     buffer = malloc(ref->ref_size);
     if (!buffer) {
-	dico_log(L_ERR, errno, "output_def");
+        DICO_LOG_ERRNO();
 	return 1;
     }
     
