@@ -1,5 +1,5 @@
 /* This file is part of GNU Dico.
-   Copyright (C) 2012-2016 Sergey Poznyakoff
+   Copyright (C) 2012-2019 Sergey Poznyakoff
 
    GNU Dico is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -14,9 +14,7 @@
    You should have received a copy of the GNU General Public License
    along with GNU Dico.  If not, see <http://www.gnu.org/licenses/>. */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+#include <config.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <fcntl.h>
@@ -116,12 +114,12 @@ gcide_idx_file_open(const char *name, size_t cachesize)
     
     file = calloc(1, sizeof(*file));
     if (!file) {
-	dico_log(L_ERR, errno, "gcide_idx_file_open");
+        DICO_LOG_ERRNO();
 	return NULL;
     }
     file->name = strdup(name);
     if (!file->name) {
-	dico_log(L_ERR, errno, "gcide_idx_file_open");
+        DICO_LOG_ERRNO();
 	free(file);
 	return NULL;
     }
@@ -168,7 +166,7 @@ _cache_alloc(struct gcide_idx_file *file)
     if (!file->cache) {
 	file->cache = calloc(file->cache_size, sizeof(file->cache[0]));
 	if (!file->cache) {
-	    dico_log(L_ERR, ENOMEM, "gcide _cache_alloc");
+            DICO_LOG_ERRNO();
 	    return NULL;
 	}
     }
@@ -178,12 +176,12 @@ _cache_alloc(struct gcide_idx_file *file)
 	    return file->cache[file->cache_used - 1];
 	cp = calloc(1, sizeof(*cp));
 	if (!cp) {
-	    dico_log(L_ERR, ENOMEM, "gcide _cache_alloc");
+            DICO_LOG_ERRNO();
 	    return NULL;
 	}
 	cp->page = malloc(file->header.ihdr_pagesize);
 	if (!cp->page) {
-	    dico_log(L_ERR, ENOMEM, "gcide _cache_alloc");
+            DICO_LOG_ERRNO();
 	    free(cp);
 	    return NULL;
 	}
@@ -413,7 +411,7 @@ gcide_idx_locate(struct gcide_idx_file *file, char *headword, size_t hwlen)
     
     itr = malloc(sizeof(*itr));
     if (!itr) {
-	dico_log(L_ERR, errno, "gcide_idx_locate");
+        DICO_LOG_ERRNO();
 	return NULL;
     }
     if (hwlen) {
@@ -424,7 +422,7 @@ gcide_idx_locate(struct gcide_idx_file *file, char *headword, size_t hwlen)
 	itr->headword = strdup(headword);
 
     if (!itr->headword) {
-	dico_log(L_ERR, errno, "gcide_idx_locate");
+        DICO_LOG_ERRNO();
 	free(itr);
 	return NULL;
     }
